import Foundation
import ProjectDescription
import TSCBasic
import TuistCore
import TuistGraph
import TuistSupport
import XCTest

@testable import TuistLoader
@testable import TuistSupportTesting

final class AutogenerationOptionsManifestMapperTests: TuistUnitTestCase {
    private typealias Manifest = ProjectDescription.Config.GenerationOptions.AutogenerationOptions

    func test_disableAutogeneratedSchemes_is_mapped_correctly() throws {
        // Given
        let manifest: ProjectDescription.Config.GenerationOptions = .disableAutogeneratedSchemes
        let temporaryPath = try temporaryPath()
        let generatorPaths = GeneratorPaths(manifestDirectory: temporaryPath)

        // When
        let got = try TuistGraph.Config.GenerationOption.from(manifest: manifest, generatorPaths: generatorPaths)

        // Then
        XCTAssertEqual(TuistGraph.Config.GenerationOption.autogenerationOptions(.disabled), got)
    }

    func test_from_returnsTheCorrectValue_whenManifestIncludesAllOptions() throws {
        // Given
        let manifest: Manifest = .enabled([.parallelizable, .randomExecutionOrdering])

        // When
        let got = try TuistGraph.AutogenerationOptions.from(manifest: manifest)

        // Then
        XCTAssertEqual(.enabled([.parallelizable, .randomExecutionOrdering]), got)
    }

    func test_from_returnsTheCorrectValue_whenManifestIsEmpty() throws {
        // Given
        let manifest: Manifest = .enabled([])

        // When
        let got = try TuistGraph.AutogenerationOptions.from(manifest: manifest)

        // Then
        XCTAssertEqual(.enabled([]), got)
    }
}
